@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------------------- */
 
 /*
- * Copyright 2007-2020 GRAHAM DUMPLETON
+ * Copyright 2007-2021 GRAHAM DUMPLETON
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -96,7 +96,7 @@ PyObject *newLogWrapperObject(PyObject *buffer)
     args = Py_BuildValue("(OssOOO)", buffer, "utf-8", "replace",
                          Py_None, Py_True, Py_True);
 
-    wrapper = PyEval_CallObject(object, args);
+    wrapper = PyObject_CallObject(object, args);
 
     Py_DECREF(args);
     Py_DECREF(object);
@@ -223,7 +223,7 @@ static PyObject *Log_isatty(LogObject *self, PyObject *args)
     return Py_False;
 }
 
-static void Log_queue(LogObject *self, const char *msg, int len)
+static void Log_queue(LogObject *self, const char *msg, Py_ssize_t len)
 {
     const char *p = NULL;
     const char *q = NULL;
@@ -330,7 +330,7 @@ static void Log_queue(LogObject *self, const char *msg, int len)
 static PyObject *Log_write(LogObject *self, PyObject *args)
 {
     const char *msg = NULL;
-    int len = -1;
+    Py_ssize_t len = -1;
 
     WSGIThreadInfo *thread_info = NULL;
 
@@ -662,7 +662,7 @@ void wsgi_log_python_error(request_rec *r, PyObject *log,
             Py_INCREF(o);
             args = Py_BuildValue("(OOOOO)", type, value, traceback,
                                  Py_None, log);
-            result = PyEval_CallObject(o, args);
+            result = PyObject_CallObject(o, args);
             Py_DECREF(args);
             Py_DECREF(o);
         }
